#!/usr/bin/python3

import json
import os
import matplotlib.pyplot as plt
from pathlib import Path
import numpy as np

#parentDir = "benchmarksVettingBadHandpicked/graphsHandpicked"
#parentDir = "benchmarksOtherEncoderConfigs/graphs"
#parentDir = "benchmarksQuteVsCqesto/graphs"
#parentDir = "benchmarks/graphs"
#parentDir = "benchmarksVettingVsEncoding/graphs"
parentDir = "benchmarksNoOverlappingTest/graphs"

def hourStringToSeconds(string):
	m, sc = string.split(':')
	s, c = sc.split('.')
	return 60*1000* int(m) + 1000 * int(s) + 10 * int(c)

def resultsForConfig(names):
	results = {}

	for name in names:
		results[name] = []

	numChoosable = 0
	numNotChoosable = 0
	lastFile = ""
	for dirname in sorted(os.listdir(parentDir)):
		graphChoosable = None
		summary = f"{dirname}: "
		for filename in sorted(os.listdir(os.path.join(parentDir, dirname))):
			for name in names:
				if name + ".json" in filename:
					localRes = json.loads(Path(os.path.join(parentDir, dirname, filename)).read_text())
					if localRes["terminated"]:
						if graphChoosable == None:
							graphChoosable = localRes["truthValue"]
						else:
							if graphChoosable != localRes["truthValue"]: print("ERROR: no agreement on choosability")
							#else: print("agreement on choosability")
						results[name].append(localRes["elapsed"])
						summary += f"{name} took {localRes['elapsed']}({hourStringToSeconds(localRes['elapsed'])/1000}); "
					else:
						summary += f"{name} didnt terminate; "
		if graphChoosable == True:
			numChoosable += 1
		elif graphChoosable == False:
			numNotChoosable += 1
			
		print(summary)
	
	print(f"found {numChoosable} choosable instances and {numNotChoosable} not choosable instances")


	return results

#importedResults = resultsForConfig(["EncoderPure", "EncoderQuteSubsets", "EncoderSubsets", "VettingMethod"])
#importedResults = resultsForConfig(["EncoderPure", "EncoderNoOverlappingOptimization","EncoderNoLazyOptimization", "EncoderNoLazyOptimizationAllColored", "EncoderNoLazyOptimizationOverlapping", "EncoderNoExactlyOneOptimization"])
importedResults = resultsForConfig(["EncoderNoLazyOptimization", "EncoderNoLazyNoOverlappingOptimization"])
#importedResults = resultsForConfig(["testQuteNoLazy", "testCqestoNoLazy"])
colorMap = {
	"EncoderPure": "red",
	"EncoderSubsets": "blue",
	"EncoderQuteSubsets": "orange",
	"VettingMethod": "green",
	"EncoderNoOverlappingOptimization": "yellow",
	"EncoderNoLazyOptimization": "magenta",
	"EncoderNoLazyNoOverlappingOptimization": "blue",
	"EncoderNoLazyOptimizationAllColored": "black",
	"EncoderNoLazyOptimizationOverlapping": "orange",
	"EncoderNoExactlyOneOptimization": "cyan",
	"testQuteNoLazy" : "blue",
	"testCqestoNoLazy" : "orange"

}
markerMap = {
	"EncoderPure": "s",
	"EncoderSubsets": "+",
	"EncoderQuteSubsets": "o",
	"VettingMethod": "x",
	"EncoderNoOverlappingOptimization": "^",
	"EncoderNoLazyNoOverlappingOptimization": "x",
	"EncoderNoLazyOptimization": "s",
	"EncoderNoLazyOptimizationAllColored": "o",
	"EncoderNoLazyOptimizationOverlapping": "x",
	"EncoderNoExactlyOneOptimization": "+",
	"testQuteNoLazy" : "+",
	"testCqestoNoLazy" : "x"
}

for name in importedResults.keys():
	print(f"{name} managed to solve {len(importedResults[name])} within the stipulated time")
	results = sorted(map(hourStringToSeconds, importedResults[name]))

	numSolved = []
	times = []
	for i, result in enumerate(results):
		numSolved.append(i)
		times.append(result)

	plt.scatter(numSolved, times, c=colorMap[name], marker=markerMap[name], label=name)
	plt.plot(numSolved, times, c=colorMap[name])

plt.legend()
plt.xlabel("number of instances solved")
plt.ylabel("runtime in ms")

plt.savefig(f"cactusPlot")
plt.show()
exit()



