#!/usr/bin/python3
import igraph as ig

def lazyChoosable(graph: ig.Graph, listsizes):
	nonLazyVertices = []
	foundLazy = False

	for i,(vertex,listsize) in enumerate(zip(graph.vs, listsizes)):
		if(vertex.degree() >= listsize):
			nonLazyVertices.append(i)
		else:
			foundLazy = True

	if foundLazy:
		graph = graph.subgraph(nonLazyVertices)
		listsizes = [elem for i, elem in enumerate(listsizes) if i in nonLazyVertices]

		return lazyChoosable(graph, listsizes)
	else:
		print("graph after lazy choosable")
		print(graph)
		return (graph, listsizes)

def cliquesOfTwins(graph: ig.Graph, listsizes, maxCliquesize):
	for cliquesize in range(1, maxCliquesize+1):

		for clique in graph.cliques(cliquesize,cliquesize):
			reducible = True

			#check, if the listsizes satisfy criteria
			cliqueListsizes = []
			for vertex in clique:
				cliqueListsizes.append(listsizes[vertex])
			for i,listsize in enumerate(sorted(cliqueListsizes)):
				#if smaller, trivial counterexamples exist
				if listsize < i + 1:
					return (graph, listsizes, False)
				#if bigger, they are not reducible
				if listsize > cliquesize:
					reducible = False
					break

			if not reducible:
				continue
			
			#check if all vertices in clique are twins
			for v1 in clique:
				for v2 in clique:
					v1Neighbors = graph.neighbors(v1)
					v2Neighbors = graph.neighbors(v2)

					for v1Neighbor in v1Neighbors:
						if not v1Neighbor == v2 and not v1Neighbor in v2Neighbors:
							reducible = False
					
					for v2Neighbor in v2Neighbors:
						if not v2Neighbor == v1 and not v2Neighbor in v1Neighbors:
							reducible = False

			if reducible:
				print("found reducible clique:")
				print(clique)
				print("corresponding graph:")
				print(graph)
				cliqueList = list(clique)
				#since the vertices in the neighborhood all get removed, and the vertices are twins, only the neighborhood of one vertex needs to be considered
				neighborhood = graph.neighbors(cliqueList[0])

				for neighbor in neighborhood:
					listsizes[neighbor] -= cliquesize

				listsizes = [elem for i,elem in enumerate(listsizes) if not i in cliqueList]
				otherVertices = [vertex for vertex in range(graph.vcount()) if not vertex in cliqueList]

				#graph without the reducible clique
				graph = graph.subgraph(otherVertices)

				return cliquesOfTwins(graph, listsizes, maxCliquesize)


	return (graph, listsizes, True)

def connectedComponents(graph:ig.Graph, listsizes):
	graph.vs["listsizes"] = listsizes

	components = graph.components().subgraphs()
	print(components)

	return [(component, component.vs["listsizes"]) for component in components]

#g,ls = lazyChoosable(ig.Graph([(0,1),(1,2),(2,3)]), [1,2,3,4])
#print(g)
#g,ls = cliquesOfTwins(ig.Graph([(0,1),(0,2),(0,3),(1,2),(1,3),(2,3),(3,4)]), [2,2,3,5,3], 3)
#res = connectedComponents(ig.Graph([(0,1),(2,3),(3,4),(5,6)]), [0,1,2,3,4,5,6])
#for (g,ls) in res:
#	print(g)
#	print(ls)