#!/usr/bin/python3
from typing import Iterable
from enum import Enum



class Literal:
	def __init__(self, truthValue: bool, variable: str):
		self.truthValue = truthValue
		self.variable = variable

	def __str__(self):
		if self.truthValue:
			return self.variable
		else:
			return "-" + self.variable

class GateConnective(Enum):
	AND = "and"
	OR = "or"
	XOR = "xor"		#only have two literals, need for checking?
	ITE = "ite"		#only have three literals, need for checking?

class Quantifier(Enum):
	FORALL = "forall"
	EXISTS = "exists"

class Gate:
	def __init__(self, variable: str, connective: GateConnective, literals: Iterable[Literal]):
		self.variable = variable
		self.connective = connective
		self.literals = literals

	def __str__(self):
		return self.variable + " = " + self.connective.value + "(" + ', '.join(map(str, self.literals)) + ")\n"

class QBlock:
	def __init__(self, quantifier: Quantifier, variables: Iterable[str]):
		self.quantifier = quantifier
		self.variables = variables

	def __str__(self):
		return self.quantifier.value + "(" + ", ".join(map(str, self.variables)) + ")\n"

class FreeBlock:
	def __init__(self, variables: Iterable[str]):
		self.variables = variables

	def __str__(self):
		if len(self.variables) == 0:
			return ""
		else:
			return "free(" + ", ".join(map(str, self.variables)) + ")\n"

class OutputStatement:
	def __init__(self, lit: str):
		self.lit = lit

	def __str__(self):
		return "output(" + self.lit + ")\n"



class QCIREncoding:
	def __init__(self, freeBlock: FreeBlock, qBlocks: Iterable[QBlock], outputStatement: OutputStatement, gateStatements: Iterable[Gate]):
		self.freeBlock = freeBlock
		self.qBlocks = qBlocks
		self.outputStatement = outputStatement
		self.gateStatements = gateStatements

	def __str__(self):
		return "#QCIR-14\n" + str(self.freeBlock) + "".join(map(str, self.qBlocks)) + str(self.outputStatement) + "".join(map(str, self.gateStatements))


def test():
	x = Literal(True, "x")
	y = Literal(False, "y")
	gate = Gate("z", GateConnective.AND, [x,y])
	print(gate)

	testFree = FreeBlock([])
	testForall = QBlock(Quantifier.FORALL, ["v1"])
	testExists = QBlock(Quantifier.EXISTS, ["v2", "v3"])

	testOutput = OutputStatement("g3")

	testGate1 = Gate("g1", GateConnective.AND, [Literal(True, "v1"), Literal(True, "v2")])
	testGate2 = Gate("g2", GateConnective.AND, [Literal(False, "v1"), Literal(False, "v2"), Literal(True, "v3")])
	testGate3 = Gate("g3", GateConnective.OR, [Literal(True, "g1"), Literal(True, "g2")])

	encoding = QCIREncoding(testFree, [testForall, testExists], testOutput, [testGate1, testGate2, testGate3])
	print(str(encoding))
