#!/usr/bin/python3

import igraph as ig
import random as rnd
import subprocess
import sys
import os
from qcir import *
import encoder as ec
import preprocessor as pp
import math

def clusterdWithThreshold(vcount, threshold):
	edges = []
	points = [(rnd.random(), rnd.random()) for i in range(vcount)]

	for i in range(vcount):
		for j in range(i+1, vcount):
			(x1,y1) = points[i]
			(x2,y2) = points[j]
			if math.sqrt(math.pow(x1-x2, 2) * math.pow(y1-y2, 2)) < threshold:
				edges.append((i,j))
	
	return ig.Graph(edges)
	

def randomGraphMeanDegree(vcount, meanDegree):
	edges = []

	for i in range(vcount):
		for j in range(i+1, vcount):
			if rnd.random() < meanDegree/(vcount - 1):
				edges.append((i,j))
	
	return ig.Graph(edges)

def randomGraph(vcount, threshold):
	edges = []

	for i in range(vcount):
		for j in range(i+1, vcount):
			if rnd.random() < threshold:
				edges.append((i,j))
	
	return ig.Graph(edges)

def testRandomGraphs():
	for i in range(0,20):
		graph = randomGraphMeanDegree(10, 4)
		print("testing the following graph")
		print(graph)
		for j in range(2,6):
			print(f"listsize: {j}")
			(lazyChoosableGraph, lazyListSizes) = pp.lazyChoosable(graph, [j]*10)
			if lazyChoosableGraph.vcount() == 0:
				print("can be colored since preprocessing yields emty graph")
				continue
			(cliquesPrunedGraph, prunedListSizes, coloringPossible) = pp.cliquesOfTwins(lazyChoosableGraph, lazyListSizes, 10)

			if not coloringPossible:
				print("can not be colored due to clique being too big")
				continue
			
			if cliquesPrunedGraph.vcount() == 0:
				print("can be colored since preprocessing yields emty graph")
				continue

			cliquesPrunedGraph.vs["listsize"] = prunedListSizes

			filename = ".temp/randomGraphTest.qcir"
			with open(filename, "w") as f:
				f.write(str(ec.basicEncoding(cliquesPrunedGraph, lazyOptimizationAllColored=True, lazyOptimizationOverlapping=False)))
			
			finishedPorocess = subprocess.run(["timeout", "300s", "time", "./qute", filename], capture_output=True)
			print(finishedPorocess.stdout)
			print(finishedPorocess.stderr)
			os.remove(filename)

def main():
	testRandomGraphs()

if __name__ == "__main__":
	main()