#!/usr/bin/python3

from qcir import *
import igraph as ig
from encoder import *


def square(oldList):
	newList = []

	for i in range(len(oldList)):
		if not oldList[i] in newList:
			newList.append(oldList[i])
		
		(a,b) = oldList[i]
		for j in range(i,len(oldList)):
			(c,d) = oldList[j]
			if a == c:
				if not (b,d) in newList and b != d:
					newList.append((b,d))
			if a == d:
				if not (b,c) in newList and b != c:
					newList.append((b,c))
			if b == c:
				if not (a,d) in newList and a != d:
					newList.append((a,d))
			if b == d:
				if not (a,c) in newList and a != c:
					newList.append((a,c))

	return newList

def printBipartiteEncoding():
	#this is K_2,4 which is known not to be 2-choosable
	g = ig.Graph(edges=[(0,1),(0,2),(0,3),(0,4),(5,1),(5,2),(5,3),(5,4)])
	g.vs["listsize"] = [2]*6
	g["maxColorabilityClasses"] = 4
	#g.vs["listsize"] = [2]*5+[3]

	#runtime with colorability classes in order and overlapping: 33s
	#g = ig.Graph(edges=[(0,1),(0,2),(0,3),(0,4),(5,1),(5,2),(5,3),(5,4),(1,6),(2,6),(3,6),(4,6),(1,7),(2,7),(3,7),(4,7)])
	#g.vs["listsize"] = [2]*4+[3]*4

	#this graph has a 7 th vertex that is connected with the 4 middle vertices
	#g = ig.Graph(edges=[(0,1),(0,2),(0,3),(0,4),(5,1),(5,2),(5,3),(5,4),(1,6),(2,6),(3,6),(4,6)])
	#g.vs["listsize"] = [3]*7

	print(basicEncoding(g, lazyOptimizationAllColored=True, lazyOptimizationOverlapping=False))

def printNielsenGraphEncoding():
	#3∗∗5∗3: runtime > 10 m
	graphFromNielsenList =  [(0,1),(0,2),(1,2),(1,3),(3,4),(4,5),(4,6),(6,7),(7,8),(5,8),(5,9),(9,10),(9,11),(10,11)]
	g = ig.Graph(edges=square(graphFromNielsenList))
	g.vs["listsize"] = [3,4,3,4,5,6,3,2,3,5,3,3]

	#3∗3: runtime 0.03s
	#graphFromNielsenList = [(0,1),(0,2),(1,2),(1,3),(3,4),(4,5),(4,6),(5,6)]

	#3**5*3 with some vertices removed
	#graphFromNielsenList =  [(0,1),(0,2),(1,2),(1,3),(3,4),(4,5),(4,6),(6,7),(7,8)]
	#runtime without (9,10): 0.6s, without (9,11): 1.1s
	#raphFromNielsenList =  [(0,1),(0,2),(1,2),(1,3),(3,4),(4,5),(4,6),(6,7),(7,8),(5,8),(5,9)]
	#note that replacing (1,3) with (2,3) reduces runtime significantly, since 0 1 2 would be lazy choosable from the beginning
	#graphFromNielsenList =  [(0,1),(0,2),(1,2),(2,3),(3,4),(4,5),(4,6),(6,7),(7,8),(5,8),(9,10),(9,11),(10,11)]
	#g = ig.Graph(edges=square(graphFromNielsenList))

	print(encodingOnlyColorabilityClasses(g, [[0], [0,1]]))
	print(basicEncoding(g, lazyOptimizationAllColored=True, lazyOptimizationOverlapping=True))

def printAlteredNielsenGraphEncoding():
	#original Graph and listsize
	#graphFromNielsenList =  [(0,1),(0,2),(1,2),(1,3),(3,4),(4,5),(4,6),(6,7),(7,8),(5,8),(5,9),(9,10),(9,11),(10,11)]
	#g = ig.Graph(edges=square(graphFromNielsenList))
	#g.vs["listsize"] = [3,4,3,4,5,6,3,2,5,3,3]

	#altered so that everything before 5,8 does not degenerate
	#graphFromNielsenList =  [(0,1),(0,2),(1,2),(1,3),(3,4),(4,5),(4,6)]
	#g = ig.Graph(edges=square(graphFromNielsenList))
	#g.vs["listsize"] = [3,4,3,4,5,3,3,2,2,5,3,3]
	#g["maxColorabilityClasses"] = 6
	
	graphFromNielsenList =  [(0,1),(0,2),(1,2),(2,3),(3,4),(4,5),(4,6),(6,7),(7,8),(5,8),(5,9)]
	f = [3,3,4,3,4,5,6,2,2,2]
	g = ig.Graph(edges=square(graphFromNielsenList))
	g.vs["listsize"] = f
	g["maxColorabilityClasses"] = 8
	print(basicEncoding(g, lazyOptimizationAllColored=True, lazyOptimizationOverlapping=False))


def printInterestingGraphFromNauty():
	graphFromNauty = ig.Graph([[0, 5], [0, 6], [0, 7], [1, 5], [1, 6], [1, 7], [2, 5], [2, 6], [2, 7], [3, 5], [3, 6], [3, 7], [4, 7]])
	graphFromNauty.vs["listsize"] = [3]*8
	graphFromNauty["maxColorabilityClasses"] = 8

	print(basicEncoding(graphFromNauty, lazyOptimizationAllColored=True, lazyOptimizationOverlapping=False))

def printVettingBadEncoding1():
	graphVettingBadList = [
		(0,2),(0,5),(0,6),(0,8),
		(1,3),(1,4),(1,7),(1,8),
		(2,3),(2,4),(2,5),
		(3,6),(3,7),
		(4,6),(4,9),
		(5,7),(5,8),
		(6,9),
		(7,9),
		(8,9)]
	subsets = [
	[0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
	[1, 2, 3, 4, 5, 6, 7, 8, 9],
	[0, 1, 3, 4, 5, 6, 7, 8, 9],
	[1, 3, 4, 5, 6, 7, 8, 9],
	[0, 3, 4, 5, 6, 7, 8, 9],
	[0, 1, 2, 4, 5, 6, 7, 8, 9],
	[1, 2, 4, 5, 6, 7, 8, 9],
	[3, 5, 6, 7, 8, 9],
	[0, 1, 2, 3, 4, 6, 7, 8, 9],
	[1, 2, 3, 4, 6, 7, 8, 9],
	[0, 1, 2, 3, 4, 5, 7, 8, 9],
	[1, 2, 3, 4, 5, 7, 8, 9],
	[0, 2, 3, 4, 5, 7, 8, 9],
	[0, 2, 3, 5, 7, 8, 9],
	[0, 2, 5, 7, 8, 9],
	[0, 5, 7, 8, 9],
	[5, 7, 8, 9],
	[1, 3, 7, 8, 9],
	[3, 7, 8, 9],
	[0, 2, 7, 8, 9],
	[0, 1, 2, 3, 4, 5, 6, 8, 9],
	[1, 2, 3, 4, 5, 6, 8, 9],
	[1, 2, 4, 5, 6, 8, 9],
	[0, 2, 4, 5, 6, 8, 9],
	[0, 4, 5, 6, 8, 9],
	[4, 5, 6, 8, 9],
	[1, 2, 3, 5, 6, 8, 9],
	[0, 2, 3, 5, 6, 8, 9],
	[3, 5, 6, 8, 9],
	[0, 5, 6, 8, 9],
	[5, 6, 8, 9],
	[1, 4, 6, 8, 9],
	[4, 6, 8, 9],
	[1, 3, 6, 8, 9],
	[3, 6, 8, 9],
	[0, 2, 4, 5, 8, 9],
	[2, 4, 5, 8, 9],
	[0, 2, 3, 5, 8, 9],
	[2, 3, 5, 8, 9],
	[0, 2, 5, 8, 9],
	[2, 5, 8, 9],
	[2, 3, 8, 9],
	[8, 9],
	[0, 1, 2, 3, 4, 5, 6, 7, 9],
	[1, 2, 3, 4, 5, 6, 7, 9],
	[2, 4, 5, 6, 7, 9],
	[4, 5, 6, 7, 9],
	[0, 1, 2, 3, 5, 6, 7, 9],
	[0, 2, 3, 5, 6, 7, 9],
	[0, 2, 5, 6, 7, 9],
	[2, 5, 6, 7, 9],
	[0, 5, 6, 7, 9],
	[5, 6, 7, 9],
	[1, 3, 4, 6, 7, 9],
	[3, 4, 6, 7, 9],
	[1, 4, 6, 7, 9],
	[1, 3, 6, 7, 9],
	[3, 6, 7, 9],
	[2, 4, 5, 7, 9],
	[0, 2, 5, 7, 9],
	[2, 5, 7, 9],
	[0, 5, 7, 9],
	[2, 3, 4, 7, 9],
	[1, 3, 4, 7, 9],
	[2, 3, 7, 9],
	[7, 9],
	[0, 2, 4, 5, 6, 9],
	[2, 4, 5, 6, 9],
	[0, 2, 5, 6, 9],
	[2, 5, 6, 9],
	[2, 3, 4, 6, 9],
	[1, 3, 4, 6, 9],
	[2, 4, 6, 9],
	[4, 6, 9],
	[2, 3, 6, 9],
	[6, 9],
	[4, 9],
	[9],
	[0, 1, 2, 3, 4, 5, 6, 7, 8],
	[1, 2, 3, 4, 5, 6, 7, 8],
	[0, 1, 3, 5, 6, 7, 8],
	[0, 3, 5, 6, 7, 8],
	[3, 5, 6, 7, 8],
	[0, 1, 3, 6, 7, 8],
	[0, 1, 2, 3, 4, 5, 7, 8],
	[1, 2, 4, 5, 7, 8],
	[0, 1, 2, 3, 5, 7, 8],
	[0, 1, 3, 5, 7, 8],
	[1, 3, 5, 7, 8],
	[0, 3, 5, 7, 8],
	[3, 5, 7, 8],
	[0, 1, 5, 7, 8],
	[0, 5, 7, 8],
	[0, 1, 2, 3, 7, 8],
	[1, 3, 7, 8],
	[0, 1, 2, 4, 5, 6, 8],
	[0, 2, 3, 5, 6, 8],
	[0, 3, 5, 6, 8],
	[3, 5, 6, 8],
	[0, 5, 6, 8],
	[3, 6, 8],
	[0, 1, 2, 4, 5, 8],
	[1, 2, 4, 5, 8],
	[0, 1, 2, 3, 5, 8],
	[1, 2, 3, 5, 8],
	[0, 2, 3, 5, 8],
	[2, 3, 5, 8],
	[0, 2, 5, 8],
	[0, 5, 8],
	[5, 8],
	[1, 2, 3, 8],
	[1, 8],
	[0, 8],
	[8],
	[2, 4, 5, 6, 7],
	[0, 3, 5, 6, 7],
	[3, 5, 6, 7],
	[1, 3, 4, 6, 7],
	[1, 3, 6, 7],
	[1, 2, 4, 5, 7],
	[0, 1, 2, 3, 5, 7],
	[1, 2, 3, 5, 7],
	[0, 2, 3, 5, 7],
	[2, 3, 5, 7],
	[5, 7],
	[1, 2, 3, 4, 7],
	[1, 3, 7],
	[3, 7],
	[1, 7],
	[7],
	[2, 4, 5, 6],
	[0, 2, 3, 5, 6],
	[2, 3, 5, 6],
	[3, 5, 6],
	[0, 2, 5, 6],
	[4, 6],
	[2, 3, 6],
	[3, 6],
	[0, 6],
	[6],
	[0, 2, 5],
	[2, 5],
	[0, 5],
	[5],
	[2, 4],
	[1, 4],
	[2, 3],
	[1, 3],
	[3],
	[0, 2],
	[2],
	[]
	]
	graphVettingBad = ig.Graph(graphVettingBadList)
	f = [2, 2, 3, 3, 2, 3, 3, 3, 3, 3]
	graphVettingBad.vs[:]["listsize"] = f
	graphVettingBad.vs[:]["originalIndex"] = range(0,graphVettingBad.vcount())
	print(encodingOnlyColorabilityClasses(graphVettingBad,subsets))


def printVettingBadEncoding2():
	graphVettingBadList = [
		(0,2),(0,5),(0,6),(0,8),
		(1,3),(1,4),(1,7),(1,8),
		(2,3),(2,4),(2,5),
		(3,6),(3,7),
		(4,6),(4,9),
		(5,7),(5,8),
		(6,9),
		(7,9),
		(8,9)]
	
	subsets = [
	[0, 1, 3, 4, 5, 6, 7, 8, 9],
	[1, 3, 4, 5, 6, 7, 8, 9],
	[0, 3, 4, 5, 6, 7, 8, 9],
	[3, 4, 5, 6, 7, 8, 9],
	[4, 5, 6, 7, 8, 9],
	[0, 5, 6, 7, 8, 9],
	[5, 6, 7, 8, 9],
	[1, 3, 4, 6, 7, 8, 9],
	[0, 1, 2, 3, 4, 5, 7, 8, 9],
	[1, 2, 3, 4, 5, 7, 8, 9],
	[0, 2, 3, 4, 5, 7, 8, 9],
	[0, 3, 4, 5, 7, 8, 9],
	[4, 5, 7, 8, 9],
	[0, 1, 2, 3, 5, 7, 8, 9],
	[0, 5, 7, 8, 9],
	[5, 7, 8, 9],
	[1, 3, 7, 8, 9],
	[3, 7, 8, 9],
	[7, 8, 9],
	[0, 1, 2, 3, 4, 5, 6, 8, 9],
	[1, 2, 3, 4, 5, 6, 8, 9],
	[0, 2, 3, 4, 5, 6, 8, 9],
	[1, 3, 4, 5, 6, 8, 9],
	[0, 4, 5, 6, 8, 9],
	[4, 5, 6, 8, 9],
	[0, 5, 6, 8, 9],
	[5, 6, 8, 9],
	[1, 4, 6, 8, 9],
	[0, 4, 6, 8, 9],
	[4, 6, 8, 9],
	[1, 3, 6, 8, 9],
	[3, 6, 8, 9],
	[1, 2, 3, 4, 5, 8, 9],
	[0, 2, 3, 4, 5, 8, 9],
	[0, 2, 4, 5, 8, 9],
	[2, 4, 5, 8, 9],
	[0, 4, 5, 8, 9],
	[4, 5, 8, 9],
	[0, 2, 5, 8, 9],
	[0, 2, 4, 8, 9],
	[8, 9],
	[0, 1, 2, 3, 4, 5, 6, 7, 9],
	[0, 2, 3, 4, 5, 6, 7, 9],
	[2, 3, 4, 5, 6, 7, 9],
	[0, 1, 3, 4, 5, 6, 7, 9],
	[1, 3, 4, 5, 6, 7, 9],
	[0, 2, 4, 5, 6, 7, 9],
	[2, 4, 5, 6, 7, 9],
	[4, 5, 6, 7, 9],
	[0, 1, 2, 3, 5, 6, 7, 9],
	[0, 5, 6, 7, 9],
	[5, 6, 7, 9],
	[1, 3, 4, 6, 7, 9],
	[3, 4, 6, 7, 9],
	[1, 4, 6, 7, 9],
	[4, 6, 7, 9],
	[1, 3, 6, 7, 9],
	[3, 6, 7, 9],
	[2, 4, 5, 7, 9],
	[4, 5, 7, 9],
	[0, 2, 5, 7, 9],
	[2, 3, 4, 7, 9],
	[1, 3, 4, 7, 9],
	[3, 4, 7, 9],
	[7, 9],
	[0, 2, 4, 5, 6, 9],
	[2, 4, 5, 6, 9],
	[0, 4, 5, 6, 9],
	[4, 5, 6, 9],
	[0, 2, 5, 6, 9],
	[2, 3, 4, 6, 9],
	[1, 3, 4, 6, 9],
	[3, 4, 6, 9],
	[0, 2, 4, 6, 9],
	[4, 6, 9],
	[6, 9],
	[0, 2, 4, 5, 9],
	[2, 4, 5, 9],
	[4, 9],
	[9],
	[0, 1, 3, 4, 5, 6, 7, 8],
	[4, 5, 6, 7, 8],
	[0, 3, 5, 6, 7, 8],
	[3, 5, 6, 7, 8],
	[0, 5, 6, 7, 8],
	[5, 6, 7, 8],
	[1, 3, 4, 6, 7, 8],
	[0, 1, 3, 6, 7, 8],
	[0, 1, 2, 3, 4, 5, 7, 8],
	[0, 1, 3, 4, 5, 7, 8],
	[1, 4, 5, 7, 8],
	[4, 5, 7, 8],
	[0, 1, 3, 5, 7, 8],
	[1, 3, 5, 7, 8],
	[0, 3, 5, 7, 8],
	[3, 5, 7, 8],
	[5, 7, 8],
	[0, 1, 2, 3, 4, 7, 8],
	[1, 3, 7, 8],
	[0, 1, 2, 3, 4, 5, 6, 8],
	[0, 4, 5, 6, 8],
	[4, 5, 6, 8],
	[0, 1, 2, 3, 5, 6, 8],
	[0, 2, 3, 5, 6, 8],
	[0, 3, 5, 6, 8],
	[3, 5, 6, 8],
	[0, 5, 6, 8],
	[0, 1, 3, 6, 8],
	[0, 6, 8],
	[0, 1, 2, 3, 4, 5, 8],
	[1, 2, 3, 4, 5, 8],
	[0, 1, 2, 4, 5, 8],
	[1, 2, 4, 5, 8],
	[0, 2, 4, 5, 8],
	[0, 1, 2, 3, 5, 8],
	[1, 2, 3, 5, 8],
	[0, 2, 3, 5, 8],
	[1, 3, 5, 8],
	[0, 5, 8],
	[5, 8],
	[1, 8],
	[0, 8],
	[8],
	[4, 5, 6, 7],
	[0, 3, 5, 6, 7],
	[3, 5, 6, 7],
	[1, 3, 4, 6, 7],
	[3, 4, 6, 7],
	[1, 4, 6, 7],
	[0, 1, 3, 6, 7],
	[3, 6, 7],
	[0, 1, 2, 3, 4, 5, 7],
	[0, 1, 2, 3, 5, 7],
	[0, 2, 3, 5, 7],
	[2, 3, 5, 7],
	[0, 1, 3, 5, 7],
	[1, 3, 5, 7],
	[0, 2, 5, 7],
	[5, 7],
	[1, 3, 4, 7],
	[1, 3, 7],
	[3, 7],
	[1, 7],
	[7],
	[0, 2, 4, 5, 6],
	[4, 5, 6],
	[0, 2, 3, 5, 6],
	[2, 3, 4, 6],
	[4, 6],
	[3, 6],
	[0, 6],
	[6],
	[0, 2, 4, 5],
	[2, 4, 5],
	[0, 2, 5],
	[2, 5],
	[0, 5],
	[5],
	[2, 4],
	[1, 4],
	[4],
	[2, 3],
	[1, 3],
	[3],
	[0, 2],
	[]
	]
	graphVettingBad = ig.Graph(graphVettingBadList)
	f = [2, 2, 2, 3, 3, 3, 3, 3, 3, 3]
	graphVettingBad.vs[:]["listsize"] = f
	graphVettingBad.vs[:]["originalIndex"] = range(0,graphVettingBad.vcount())
	print(encodingOnlyColorabilityClasses(graphVettingBad,subsets))

def main():
	printVettingBadEncoding1()
	#printVettingBadEncoding2()
	#printBipartiteEncoding()
	#printNielsenGraphEncoding()
	#printAlteredNielsenGraphEncoding()
	#printInterestingGraphFromNauty()

if __name__ == "__main__":
    main()
